<BLOCK>
<LICENSE>

/* CONVOLUTIONAL ENCODER                                            */
/* Copyright (c) 1999, Spectrum Applications, Derwood, MD, USA      */
/* All rights reserved                                              */
/* Version 2.0 Last Modified 1999.02.17                             */


/*
 * Visit:
 * http://home.netcom.com/~chip.f/viterbi/tutorial.html
 * for latest on the Convolutional Encoder .
 */
 
 



/*
 * Converted to CAPSIM Block by Sasan Ardalan
 * Date August 15, 2001
 */


</LICENSE>
<BLOCK_NAME>
convenc
</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include <math.h>
#include "viterbi.h"
#include "poly.h"

]]>
</INCLUDES> 

<DECLARATIONS> 

    int i,j,n;
    float xval;
    float tmp;
    int  k;                    /* loop variables */
    int p, q;                  /* the upper and lower xor gate outputs */

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> shift_reg </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> sr_head </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> unencoded_data </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> m </NAME>
	</STATE>
	<STATE>
		<TYPE> int** </TYPE>
		<NAME> g </NAME>
	</STATE>
	<STATE>
		<TYPE> long </TYPE>
		<NAME> t </NAME>
	</STATE>
	<STATE>
		<TYPE> long </TYPE>
		<NAME> tt </NAME>
	</STATE>
</STATES>



<PARAMETERS>
	<PARAM>
		<DEF> Contraint Length K ( 3,5,7,9) </DEF>
		<TYPE> int </TYPE>
		<NAME> K </NAME>
		<VALUE> 7 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Frame Size </DEF>
		<TYPE> int </TYPE>
		<NAME> frameLength </NAME>
		<VALUE> 1024 </VALUE>
	</PARAM>
</PARAMETERS>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> bits </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> symbols </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 

    m=K-1;
    shift_reg= (int *) calloc(K,sizeof(int));
    if(shift_reg==NULL) {
           printf("convenc: could not allocate space for shift register\n");
           return(1);
    }
    unencoded_data =(int*)calloc( (frameLength + m+1) ,sizeof(int) );
    if(unencoded_data==NULL) {
           printf("convenc: could not allocate space for unencoded data\n");
           return(2);
    }
    g =(int**)calloc( 2 ,sizeof(int *) );
    g[0]=(int*)calloc( K ,sizeof(int) );
    g[1]=(int*)calloc( K ,sizeof(int) );
    if(g[0]==NULL || g[1]==NULL) {
           printf("convenc: could not allocate space for polynominals\n");
           return(4);
    }
    /*
     * see poly.h for g_3[][], g_5[][], g_7[][], g_9[][]
     */
    switch (K) {
      case 3:
        for(i=0; i<K; i++) {
            g[0][i]=g_3[0][i];
            g[1][i]=g_3[1][i];
        }
        break;
      case 5:
        for(i=0; i<K; i++) {
            g[0][i]=g_5[0][i];
            g[1][i]=g_5[1][i];
        }
        break;
      case 7:
        for(i=0; i<K; i++) {
            g[0][i]=g_7[0][i];
            g[1][i]=g_7[1][i];
        }
        break;
      case 9:
        for(i=0; i<K; i++) {
            g[0][i]=g_9[0][i];
            g[1][i]=g_9[1][i];
        }
        break;
      default:
         printf("convenc: constraint K not valid:%d\n",K);
         return(3);
    }
   /* Initialize the shift register */
    for (j = 0; j < K; j++) {
        shift_reg[j] = 0;
    }
    /* To try to speed things up a little, the shift register will be operated
       as a circular buffer, so it needs at least a head pointer. It doesn't
       need a tail pointer, though, since we won't be taking anything out of
       it--we'll just be overwriting the oldest entry with the new data. */
    sr_head = 0;
    /* initialize the channel symbol output index */
    tt = 0;

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


    for(i=MIN_AVAIL();i>0; --i) {
       IT_IN(0);
       unencoded_data[t]=bits(0);
       t++;
       if(t==frameLength) {
             /* Now start the encoding process */

             /*
              * clear shift register
              */
             for(i=0; i<K; i++) shift_reg[i]=0;


             /*
              * Add zeros to end of frame
              */
             for(i=0; i<m; i++) unencoded_data[frameLength+i]=0;


             /* compute the upper and lower mod-two adder outputs, one bit at a time */
             for (n = 0; n < frameLength + m; n++) {
               shift_reg[sr_head] = *( unencoded_data + n );
               p = 0;
               q = 0;
               for (j = 0; j < K; j++) {
                        k = (j + sr_head) % K;
                        p ^= shift_reg[k] & g[0][j];
                        q ^= shift_reg[k] & g[1][j];
               }

               /* write the upper and lower xor gate outputs as channel symbols */
               if(IT_OUT(0)) {
                    KrnOverflow("convenc",0);
                    return(99);
               }


               symbols(0)=p;

               if(IT_OUT(0)) {
                    KrnOverflow("convenc",0);
                    return(99);
               }

               symbols(0)=q;


               sr_head -= 1;    /* equivalent to shifting everything right one place */
               if (sr_head < 0) /* but make sure we adjust pointer modulo K */
                      sr_head = m;

            }


          t=0;
       }

    }


]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

if(shift_reg)
       free(shift_reg);
if(unencoded_data)
       free(unencoded_data);

]]>
</WRAPUP_CODE> 



</BLOCK> 

