<BLOCK>
<LICENSE>

/*  Capsim (r) Text Mode Kernel (TMK) Block Library (Blocks)
 *  Copyright (C) 2002-2017  Silicon DSP Corporation
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    http://www.silicondsp.com
 *    Silicon DSP  Corporation
 *    Las Vegas, Nevada
 */



</LICENSE>

<BLOCK_NAME>
demapper
</BLOCK_NAME> 

<DESC_SHORT>
11a Demapper
</DESC_SHORT>


<COMMENTS>
<![CDATA[ 

/* demapper11a.s */
/******************************************************************
			demaper11a()
*******************************************************************
Programmer: 	Bin Huang	
Date:	 	    6/20/2002	

*/
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include "demap.h"
#include "ftab_80211a.h"
]]>

</INCLUDES> 


<PARAMETERS>
	<PARAM>
		<DEF> MAP (0-BPSK 1-QPSK 2-16QAM 3-64QAM </DEF>
		<TYPE> int </TYPE>
		<NAME> map </NAME>
		<VALUE> 3 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Soft=1 Hard=0 </DEF>
		<TYPE> int </TYPE>
		<NAME> soft </NAME>
		<VALUE> 1 </VALUE>
	</PARAM>
	
	
	
</PARAMETERS>



<DEFINES> 

#define N 1
#define MAP_BPSK 0
#define MAP_QPSK 1
#define MAP_16_QAM 2
#define MAP_64_QAM 3


</DEFINES> 

<DECLARATIONS> 

	int j;	

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> x_in </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> y_in </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> x_out </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> ibufs </NAME>
		<VALUE> 2 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> count </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> qamMap </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> Nbpsc </NAME>
		<VALUE> 1 </VALUE>
	</STATE>

</STATES>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> complex </TYPE>
		<NAME> x </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> out3soft </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 


   switch (map) {
       case MAP_BPSK:
              Nbpsc=1;
              qamMap=MOD_11a_BPSK;
              break;
       case MAP_QPSK:
              Nbpsc=2;
              qamMap=MOD_11a_QPSK;
              break;

       case MAP_16_QAM:
              Nbpsc=4;
              qamMap=MOD_11a_16_QAM;
              break;
       case MAP_64_QAM:
              Nbpsc=6;
              qamMap=MOD_11a_64_QAM;
              break;
       default:
              fprintf(stderr,"mapper11a: incorrect rate\n");
              return(10);
              break;

   }
   



         x_in =(float*)calloc(N, sizeof(float));
         y_in =(float*)calloc(N, sizeof(float));
         x_out =(float*)calloc(Nbpsc*N, sizeof(float));
         if(x_in == NULL || y_in == NULL || x_out == NULL)  {
             fprintf(stderr,"%s: Could not allocate space\n",STAR_NAME);
             return(3);
         }

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 

        while(IT_IN(0))
	{
		x_in[count] = x(0).re;
                y_in[count] = x(0).im;		
		count++;
	        if(count == N)
	        {
                      count = 0;
	              if(soft)
	              {
	                   /* printf("Soft demapping is being performed\n"); */
	                     soft_demap(x_in, y_in, x_out,qamMap);
	              }
	              else
	              {
	                   /*  printf("Hard decision demapping is being performed\n");	*/
	                     hard_demap(x_in, y_in, x_out,qamMap);
	              }       	
	             
	
	              for(j=0; j<Nbpsc*N; j++)
	              {
	                     IT_OUT(0);
	                     out3soft(0) = (float)x_out[j];
//printf("Demapper %f\n",(float)x_out[j]);
                             x_out[j]=0.0;  
                      }
	
	        }
	
	 }
		
	return(0);

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

         free(x_in );
         free(y_in );
         free(x_out);

]]>
</WRAPUP_CODE> 



</BLOCK> 

