<BLOCK>
<LICENSE>

/*  Capsim (r) Text Mode Kernel (TMK) Block Library (Blocks)
 *  Copyright (C) 2002-2017  Silicon DSP Corporation
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    http://www.silicondsp.com
 *    Silicon DSP  Corporation
 *    Las Vegas, Nevada
 */


</LICENSE>

<BLOCK_NAME>
mapper
</BLOCK_NAME> 

<DESC_SHORT>
 Floating point 802.11a QAM mapping function
</DESC_SHORT>



<COMMENTS>
<![CDATA[ 

/*
 File:                   mapper11a.s
 Description: Floating point 802.11a QAM mapping function, total number of
              input bits depends on rate. The output should be 48 complex symbols.
   Programmer:         Bin Huang
   Date:               8/21/2001
*/
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 
#include "ftab_80211a.h"
]]>
</INCLUDES> 

<DEFINES> 

#define MAP_BPSK 0
#define MAP_QPSK 1
#define MAP_16_QAM 2
#define MAP_64_QAM 3

#define NSIZE 1

</DEFINES> 

<DECLARATIONS> 

  int tableIndex,i,j;
  float  real, imag;
  complex cl;

</DECLARATIONS> 


<PARAMETERS>
	<PARAM>
		<DEF> MAP (0-BPSK 1-QPSK 2-16QAM 3-64QAM </DEF>
		<TYPE> int </TYPE>
		<NAME> map </NAME>
		<VALUE> 3 </VALUE>
		
		
	</PARAM>
</PARAMETERS>

<STATES>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> numberBits </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> qaminput </NAME>
	</STATE>

	<STATE>
		<TYPE> int </TYPE>
		<NAME> NBITS </NAME>
	</STATE>

	<STATE>
		<TYPE> complex* </TYPE>
		<NAME> qam </NAME>
	</STATE>

	<STATE>
		<TYPE> int</TYPE>
		<NAME> qamMap </NAME>
	</STATE>

</STATES>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> x </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> complex </TYPE>
		<NAME> output </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 


   switch (map) {
       case MAP_BPSK:
              NBITS =1;
              qamMap=MOD_11a_BPSK;
              break;
       case MAP_QPSK:
              NBITS =2;
              qamMap=MOD_11a_QPSK;
              break;

       case MAP_16_QAM:
              NBITS =4; 
              qamMap=MOD_11a_16_QAM;
              break;
       case MAP_64_QAM:
              NBITS =6;
              qamMap=MOD_11a_64_QAM;
              break;
       default:
              fprintf(stderr,"mapper: incorrect map\n");
              return(10);
              break;

   }

   if((qaminput = (int*)calloc(NBITS,sizeof(int))) == NULL ||
       (qam = (complex*)calloc(NSIZE,sizeof(complex))) == NULL) {
         fprintf(stderr,"QAM: can't allocate work space\n");
         return(4);
   }
   numberBits = 0;
   SET_CELL_SIZE_OUT(0,sizeof(complex));

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


while(IT_IN(0))
{
       qaminput[numberBits] = (int)x(0);
       numberBits++;
       if (numberBits == NBITS)
       {
           numberBits = 0;
           j = 0;
           switch(qamMap) {
              case MOD_11a_64_QAM:
                  for (i=0; i<NBITS; i+=6)
                  {
//                         tableIndex = qaminput[i]+2*qaminput[i+1]+4*qaminput[i+2]+8*qaminput[i+3]+16*qaminput[i+4]+32*qaminput[i+5];

                        tableIndex = 32*qaminput[i]+16*qaminput[i+1]+8*qaminput[i+2]+4*qaminput[i+3]+2*qaminput[i+4]+qaminput[i+5];

              
                         real = QAM_64MAPTable[0][tableIndex];
                         imag = QAM_64MAPTable[1][tableIndex];
                         qam[j].re = real;
                         qam[j].im = imag;
//                         printf("QAM 64,j,index,r,im\t%d\t%d\t%f\t%f\n",j ,tableIndex, real, imag);
                         //j++;
                   }
                   break;
             case MOD_11a_16_QAM:
                  for (i=0; i<NBITS; i+=4)
                  {
                        tableIndex = 8*qaminput[i]+4*qaminput[i+1]+2*qaminput[i+2]+qaminput[i+3];
        
                         real = QAM_16MAPTable[0][tableIndex];
                         imag = QAM_16MAPTable[1][tableIndex];
                         qam[j].re = real;
                         qam[j].im = imag;
//printf("QAM 16,j,index,r,im\t%d\t%d\t%f\t%f\n",j ,tableIndex, real, imag);
                          //j++;
                   }
                   break;
             case MOD_11a_QPSK:
                  for (i=0; i<NBITS; i+=2)
                  {
                        tableIndex = qaminput[i]+2*qaminput[i+1];
        
                         real = QPSK_MAPTable[0][tableIndex];
                         imag = QPSK_MAPTable[1][tableIndex];
                         qam[j].re = real;
                         qam[j].im = imag;
//printf("QAM QPSK,j,index,r,im\t%d\t%d\t%f\t%f\n",j ,tableIndex, real, imag);
                          //j++;
                   }
                   break;
             case MOD_11a_BPSK:
                  for (i=0; i<NBITS; i+=1)
                  {
                        tableIndex = qaminput[i];
        
                         real = BPSK_MAPTable[0][tableIndex];
                         imag = BPSK_MAPTable[1][tableIndex];
                         qam[j].re = real;
                         qam[j].im = imag;
//printf("BPSK,j,index,r,im\t%d\t%d\t%f\t%f\n",j ,tableIndex, real, imag);
                          //j++;
                   }
                   break;
              default:
                   break;
          }

          for (i=0; i<NSIZE; i++)
          {
                 if (IT_OUT(0))
                 {
                        KrnOverflow("mapper11a", 0);
                        return(99);
                 }

                 cl.re = qam[i].re;
                 cl.im = qam[i].im;
  //          printf("QAM   %f  %f\n", cl.re,cl.im);
                 output(0) = cl;

          }
          
     }
}



]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

free(qaminput);
free(qam);

]]>
</WRAPUP_CODE> 



</BLOCK> 

