<BLOCK>
<LICENSE>


/* Note the Viterbi Decoder Subroutine (separate from this block) is */
/*         sdvd                                                */
/*  
/* The Viterbi Decoder is:                                           */
/* Copyright (c) 1999, Spectrum Applications, Derwood, MD, USA       */
/* All rights reserved                                               */
/* Version 2.0 Last Modified 1999.02.17                              */

/* SOFT-DECISION VITERBI DECODER                                     */
/* Copyright (c) 1999-2011 Robert F. Fleming III, Maryland, USA      */
/* All rights reserved                                               */
/* Version 2.3 Last Modified 2011.12.13                              */


/*
 * Visit:
 * http://home.netcom.com/~chip.f/viterbi/tutorial.html
 * for latest on the Viterbi Decoder.
 */


/*
 * Converted to CAPSIM Block by Sasan Ardalan
 * Date August 15, 2001
 */

</LICENSE>
<BLOCK_NAME>
vitdec
</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include "viterbi.h"
#include "poly.h"

]]>
</INCLUDES> 

<DECLARATIONS> 

    int i,j,n;
    float xval;
    float tmp;
    int  k;                    /* loop variables */

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> shift_reg </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> sr_head </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> decodedData </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> encodedData </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> m </NAME>
	</STATE>
	<STATE>
		<TYPE> int** </TYPE>
		<NAME> g </NAME>
	</STATE>
	<STATE>
		<TYPE> long </TYPE>
		<NAME> t </NAME>
	</STATE>
	<STATE>
		<TYPE> long </TYPE>
		<NAME> tt </NAME>
	</STATE>
</STATES>



<PARAMETERS>
	<PARAM>
		<DEF> Contraint Length K ( 3,5,7,9) </DEF>
		<TYPE> int </TYPE>
		<NAME> K </NAME>
		<VALUE> 7 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Frame Size </DEF>
		<TYPE> int </TYPE>
		<NAME> frameLength </NAME>
		<VALUE> 1024 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> ES Over N0 </DEF>
		<TYPE> float </TYPE>
		<NAME> es_ovr_n0 </NAME>
		<VALUE> 0.0 </VALUE>
	</PARAM>
</PARAMETERS>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> symbols </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> bits </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 

    m=K-1;
    encodedData =(float*)calloc( (2*(frameLength + m)) ,sizeof(float) );
    if(encodedData==NULL) {
           printf("vitdec: could not allocate space for encoded  data\n");
           return(2);
    }
    decodedData =(int*)calloc( (frameLength + m) ,sizeof(int) );
    if(decodedData==NULL) {
           printf("vitdec: could not allocate space for encoded  data\n");
           return(2);
    }
    g =(int**)calloc( 2 ,sizeof(int *) );
    g[0]=(int*)calloc( K ,sizeof(int) );
    g[1]=(int*)calloc( K ,sizeof(int) );
    if(g[0]==NULL || g[1]==NULL) {
           printf("vitdec: could not allocate space for polynominals\n");
           return(4);
    }
    /*
     * see poly.h for g_3[][], g_5[][], g_7[][], g_9[][]
     */
    switch (K) {
      case 3:
        for(i=0; i<K; i++) {
            g[0][i]=g_3[0][i];
            g[1][i]=g_3[1][i];
        }
        break;
      case 5:
        for(i=0; i<K; i++) {
            g[0][i]=g_5[0][i];
            g[1][i]=g_5[1][i];
        }
        break;
      case 7:
        for(i=0; i<K; i++) {
            g[0][i]=g_7[0][i];
            g[1][i]=g_7[1][i];
        }
        break;
      case 9:
        for(i=0; i<K; i++) {
            g[0][i]=g_9[0][i];
            g[1][i]=g_9[1][i];
        }
        break;
      default:
         printf("convenc: constraint K not valid:%d\n",K);
         return(3);
    }
    /* initialize the channel symbol output index */
    tt = 0;

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


    for(i=MIN_AVAIL();i>0; --i) {
       IT_IN(0);
       encodedData[t]=symbols(0);
       t++;
       if(t==(2*(frameLength+m))) {
             /* Now start the decoding  process */
             sdvd(K,g, es_ovr_n0, 2*(frameLength+m), encodedData, decodedData);


             /*
              * output the frame.
              * skip m=K-1 last bits
              */

             for(n=0; n<frameLength; n++) {
               if(IT_OUT(0)) {
                    KrnOverflow("convenc",0);
                    return(99);
               }
               bits(0)=decodedData[n];
             }

          t=0;
       }

    }


]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 
#if 000
if(decodedData)
       free( decodedData);
if(encodedData)
       free(encodedData);
#endif

]]>
</WRAPUP_CODE> 



</BLOCK> 

