<BLOCK>
<LICENSE>
/*  Capsim (r) Text Mode Kernel (TMK) Block Library (Blocks)
 *  Copyright (C) 2002-2017  Silicon DSP Corporation
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    http://www.silicondsp.com
 *    Silicon DSP  Corporation
 *    Las Vegas, Nevada
 */


</LICENSE>

<BLOCK_NAME>
vitdec_pars80216
</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

/* Note the Viterbi Decoder Subroutine (separate from this block) is */
/*         sdvd_parms                                                */
/*  
/* The Viterbi Decoder is:                                           */
/* Copyright (c) 1999, Spectrum Applications, Derwood, MD, USA       */
/* All rights reserved                                               */
/* Version 2.0 Last Modified 1999.02.17                              */

/* SOFT-DECISION VITERBI DECODER                                     */
/* Copyright (c) 1999-2011 Robert F. Fleming III, Maryland, USA      */
/* All rights reserved                                               */
/* Version 2.3 Last Modified 2011.12.13                              */


/*
 * Visit:
 * http://home.netcom.com/~chip.f/viterbi/tutorial.html
 * for latest on the Viterbi Decoder.
 */
 

/*
 * CAPSIM Block by Sasan Ardalan
 * Date August 15, 2001
 */
 /* Modified by AymanE on 10-1-01:
  * Made parameters:
  * quantizerType, softBitsPerValue, tracebackDepthFactor.
  */
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 
#include "viterbi_parms.h"
#include "poly.h"

]]>
</INCLUDES> 

<DEFINES> 

#define MaxSize  65600

</DEFINES> 

<DECLARATIONS> 

    int i,j,n;
    float xval;
    float tmp;
    int  k;                    /* loop variables */
	double temp;

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> shift_reg </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> sr_head </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> decodedData </NAME>
	</STATE>
	<STATE>
		<TYPE> float* </TYPE>
		<NAME> encodedData </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> m </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> numberOfSym </NAME>
	</STATE>
	<STATE>
		<TYPE> int** </TYPE>
		<NAME> g </NAME>
	</STATE>
	<STATE>
		<TYPE> int** </TYPE>
		<NAME> input </NAME>
	</STATE>
	<STATE>
		<TYPE> long </TYPE>
		<NAME> t </NAME>
	</STATE>
	<STATE>
		<TYPE> long </TYPE>
		<NAME> tt </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> N_DBPS </NAME>
	</STATE>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> numberCodedBytes </NAME>
	</STATE>
	
	
	
</STATES>



<PARAMETERS>
	<PARAM>
		<DEF> Size of input data (original:bits) </DEF>
		<TYPE> int </TYPE>
		<NAME> sizeOfFrame </NAME>
		<VALUE> 747 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Contraint Length K ( 3,5,7,9) </DEF>
		<TYPE> int </TYPE>
		<NAME> K </NAME>
		<VALUE> 7 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> ES Over N0 </DEF>
		<TYPE> float </TYPE>
		<NAME> es_ovr_n0 </NAME>
		<VALUE> 0.0 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Quantizer Type (1,2, or 3) </DEF>
		<TYPE> int </TYPE>
		<NAME> quantizerType </NAME>
		<VALUE> 2 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Number of Bits Per Soft Value </DEF>
		<TYPE> int </TYPE>
		<NAME> bitsPerSoftValue </NAME>
		<VALUE> 3 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> The traceback Depth as a multiple of the constraint length </DEF>
		<TYPE> int </TYPE>
		<NAME> tracebackDepthFactor </NAME>
		<VALUE> 5 </VALUE>
	</PARAM>
	<PARAM>
		<DEF> Frame Length</DEF>
		<TYPE> int </TYPE>
		<NAME> frameLength </NAME>
		<VALUE> 1000 </VALUE>
	</PARAM>	
</PARAMETERS>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> symbols </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> float </TYPE>
		<NAME> bits </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 




    m=K-1;
   	
    encodedData =(float*)calloc(2*frameLength+100 ,sizeof(float) );
    if(encodedData==NULL) {
           printf("vitdec: could not allocate space for encoded  data\n");
           return(2);
    }
    decodedData =(int*)calloc(frameLength+100 ,sizeof(int) );
    if(decodedData==NULL) {
           printf("vitdec: could not allocate space for encoded  data\n");
           return(2);
    }
    g =(int**)calloc( 2 ,sizeof(int *) );
    g[0]=(int*)calloc( K ,sizeof(int) );
    g[1]=(int*)calloc( K ,sizeof(int) );
    if(g[0]==NULL || g[1]==NULL) {
           printf("vitdec: could not allocate space for polynominals\n");
           return(4);
    }
//  int input[TWOTOTHEM_MAX][TWOTOTHEM_MAX];          /* maps current/nxt sts to input */
    input =(int**)calloc( TWOTOTHEM_MAX ,sizeof(int *) );
	for(i=0; i<TWOTOTHEM_MAX; i++) {
         input[i]=(int*)calloc( TWOTOTHEM_MAX ,sizeof(int) );
         if(input[i]==NULL) {
              printf("vitdec: could not allocate space for input\n");
              return(4);
         }
	}
    /*
     * see poly.h for g_3[][], g_5[][], g_7[][], g_9[][]
     */
    switch (K) {
      case 3:
        for(i=0; i<K; i++) {
            g[0][i]=g_3[0][i];
            g[1][i]=g_3[1][i];
        }
        break;
      case 5:
        for(i=0; i<K; i++) {
            g[0][i]=g_5[0][i];
            g[1][i]=g_5[1][i];
        }
        break;
      case 7:
        for(i=0; i<K; i++) {
            g[0][i]=g_7[0][i];
            g[1][i]=g_7[1][i];
        }
        break;
      case 9:
        for(i=0; i<K; i++) {
            g[0][i]=g_9[0][i];
            g[1][i]=g_9[1][i];
        }
        break;
      default:
         printf("convenc: constraint K not valid:%d\n",K);
         return(3);
    }
    /* initialize the channel symbol output index */
    tt = 0;

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


    while(IT_IN(0)) {
       encodedData[t]=symbols(0);
       t++;
  //     if(t - 432*( (int)floor(t/432) ) == 0 )

 /*
       if((t == 2) && vitDone)
	     {
		    vitDone = 0;
			vitSkip = 1;
			t = 0;
			frameLength = ((int) encodedData[1])*8 + 18;   //grab the data length

         }
*/

       if(t==(2*(frameLength+m))) {
             /*printf("\n\n\n       Now start the decoding process \n"); */
             /* Now start the decoding  process */
             sdvd_parms(K,g, es_ovr_n0, 2*(frameLength+m), encodedData, decodedData,quantizerType,bitsPerSoftValue,tracebackDepthFactor,input);

             /*
              * output the frame.
              * skip m=K-1 last bits plus 18 non useful bits
              */

             for(n=0; n<frameLength-18; n++) {
               if(IT_OUT(0)) {
                    KrnOverflow("convenc",0);
                    return(99);
               }
               bits(0)=decodedData[n];
             }
          /* printf("Viterbi decoding is DONE \n"); */
          t=0;
       }

    }


]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 

if(decodedData)
       free( decodedData);
if(encodedData)
       free(encodedData);
	   
    free(g[0]);
    free(g[1]);

    free(g);
    for(i=0; i<TWOTOTHEM_MAX; i++) 
         free(input[i]);
    free(input);

]]>
</WRAPUP_CODE> 

  


</BLOCK> 

